"""

    """

import asyncio

from src.mirutil.async_req import get_reqs_and_save_async_sync as grasas


##
url = 'https://excel.codal.ir/service/Excel/GetAll/3scwkK3cwWVPTvP5QQQaQQQ2%2b9RA%3d%3d/0'

grasas([url] , ['1.html'])

##

##

##

##
